@@ -32,18 +32,17 @@
 #include <sstream>
 #include <iostream>
 #include <assert.h>
-#include <QRegExp>
 
 #include <boost/foreach.hpp>
-#include <boost/unordered_map.hpp>
 #include <boost/bind.hpp>
+#include <map>
 
 CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const AbstractNode &node)
 {
 	if (!isCached(node)) {
 		Traverser evaluate(*this, node, Traverser::PRE_AND_POSTFIX);
 		evaluate.execute();
-		assert(isCached(node));
+		return this->root;
 	}
 	return CGALCache::instance()->get(this->tree.getIdString(node));
 }
@@ -62,14 +61,16 @@ void CGALEvaluator::process(CGAL_Nef_polyhedron &target, const CGAL_Nef_polyhedr
  	if (target.dim != 2 && target.dim != 3) {
  		assert(false && "Dimension of Nef polyhedron must be 2 or 3");
  	}
-	if (src.empty()) return; // Empty polyhedron. This can happen for e.g. square([0,0])
+	if (src.isEmpty()) return; // Empty polyhedron. This can happen for e.g. square([0,0])
+	if (target.isEmpty() && op != CGE_UNION) return; // empty op <something> => empty
 	if (target.dim != src.dim) return; // If someone tries to e.g. union 2d and 3d objects
 
 	CGAL::Failure_behaviour old_behaviour = CGAL::set_error_behaviour(CGAL::THROW_EXCEPTION);
 	try {
 		switch (op) {
 		case CGE_UNION:
-			target += src;
+			if (target.isEmpty()) target = src.copy();
+			else target += src;
 			break;
 		case CGE_INTERSECTION:
 			target *= src;
@@ -82,10 +83,13 @@ void CGALEvaluator::process(CGAL_Nef_polyhedron &target, const CGAL_Nef_polyhedr
 			break;
 		}
 	}
-	catch (CGAL::Assertion_exception e) {
-		// union && difference assert triggered by testdata/scad/bugs/rotate-diff-nonmanifold-crash.scad
+	catch (const CGAL::Failure_exception &e) {
+		// union && difference assert triggered by testdata/scad/bugs/rotate-diff-nonmanifold-crash.scad and testdata/scad/bugs/issue204.scad
 		std::string opstr = op == CGE_UNION ? "union" : op == CGE_INTERSECTION ? "intersection" : op == CGE_DIFFERENCE ? "difference" : op == CGE_MINKOWSKI ? "minkowski" : "UNKNOWN";
-		PRINTF("CGAL error in CGAL_Nef_polyhedron's %s operator: %s", opstr.c_str(), e.what());
+		PRINTB("CGAL error in CGAL_Nef_polyhedron's %s operator: %s", opstr % e.what());
+
+		// Errors can result in corrupt polyhedrons, so put back the old one
+		target = src;
 	}
 	CGAL::set_error_behaviour(old_behaviour);
 }
@@ -108,7 +112,8 @@ CGAL_Nef_polyhedron CGALEvaluator::applyToChildren(const AbstractNode &node, CGA
 		if (!isCached(*chnode)) {
 			CGALCache::instance()->insert(this->tree.getIdString(*chnode), chN);
 		}
-		if (N.empty()) N = chN.copy();
+		// Initialize N on first iteration with first expected geometric object
+		if (N.isNull() && !N.isEmpty()) N = chN.copy();
 		else process(N, chN, op);
 
 		chnode->progress_report();
@@ -128,6 +133,7 @@ CGAL_Nef_polyhedron CGALEvaluator::applyHull(const CgaladvNode &node)
 		const CGAL_Nef_polyhedron &chN = item.second;
 		// FIXME: Don't use deep access to modinst members
 		if (chnode->modinst->isBackground()) continue;
+		if (chN.dim == 0) continue; // Ignore object with dimension 0 (e.g. echo)
 		if (dim == 0) {
 			dim = chN.dim;
 		}
@@ -146,9 +152,14 @@ CGAL_Nef_polyhedron CGALEvaluator::applyHull(const CgaladvNode &node)
 		}
 		else if (dim == 3) {
 			CGAL_Polyhedron P;
-			chN.p3->convert_to_Polyhedron(P);
-			std::transform(P.vertices_begin(), P.vertices_end(), std::back_inserter(points3d), 
-										 boost::bind(static_cast<const CGAL_Polyhedron::Vertex::Point_3&(CGAL_Polyhedron::Vertex::*)() const>(&CGAL_Polyhedron::Vertex::point), _1));
+			if (!chN.p3->is_simple()) {
+				PRINT("Hull() currently requires a valid 2-manifold. Please modify your design. See http://en.wikibooks.org/wiki/OpenSCAD_User_Manual/STL_Import_and_Export");
+			}
+			else {
+				chN.p3->convert_to_Polyhedron(P);
+				std::transform(P.vertices_begin(), P.vertices_end(), std::back_inserter(points3d), 
+											 boost::bind(static_cast<const CGAL_Polyhedron::Vertex::Point_3&(CGAL_Polyhedron::Vertex::*)() const>(&CGAL_Polyhedron::Vertex::point), _1));
+			}
 		}
 		chnode->progress_report();
 	}
@@ -161,7 +172,8 @@ CGAL_Nef_polyhedron CGALEvaluator::applyHull(const CgaladvNode &node)
 	}
 	else if (dim == 3) {
 		CGAL_Polyhedron P;
-		CGAL::convex_hull_3(points3d.begin(), points3d.end(), P);
+		if (points3d.size()>3)
+			CGAL::convex_hull_3(points3d.begin(), points3d.end(), P);
 		N = CGAL_Nef_polyhedron(new CGAL_Nef_polyhedron3(P));
 	}
 	return N;
@@ -236,40 +248,61 @@ Response CGALEvaluator::visit(State &state, const TransformNode &node)
 		if (!isCached(node)) {
 			// First union all children
 			N = applyToChildren(node, CGE_UNION);
+			if ( matrix_contains_infinity( node.matrix ) || matrix_contains_nan( node.matrix ) ) {
+				// due to the way parse/eval works we can't currently distinguish between NaN and Inf
+				PRINT("Warning: Transformation matrix contains Not-a-Number and/or Infinity - removing object.");
+				N.reset();
+			}
 
 			// Then apply transform
-			// If there is no geometry under the transform, N will be empty and of dim 0,
-			// just just silently ignore such nodes
-			if (N.dim == 2) {
-				// Unfortunately CGAL provides no transform method for CGAL_Nef_polyhedron2
-				// objects. So we convert in to our internal 2d data format, transform it,
-				// tesselate it and create a new CGAL_Nef_polyhedron2 from it.. What a hack!
-				
-				CGAL_Aff_transformation2 t(
-					node.matrix(0,0), node.matrix(0,1), node.matrix(0,3),
-					node.matrix(1,0), node.matrix(1,1), node.matrix(1,3), node.matrix(3,3));
-				
-				DxfData *dd = N.convertToDxfData();
-				for (size_t i=0; i < dd->points.size(); i++) {
-					CGAL_Kernel2::Point_2 p = CGAL_Kernel2::Point_2(dd->points[i][0], dd->points[i][1]);
-					p = t.transform(p);
-					dd->points[i][0] = to_double(p.x());
-					dd->points[i][1] = to_double(p.y());
+			// If there is no geometry under the transform, N will be empty
+			// just silently ignore such nodes
+			if (!N.isNull()) {
+				if (N.dim == 2) {
+					// Unfortunately CGAL provides no transform method for CGAL_Nef_polyhedron2
+					// objects. So we convert in to our internal 2d data format, transform it,
+					// tesselate it and create a new CGAL_Nef_polyhedron2 from it.. What a hack!
+					
+					Eigen::Matrix2f testmat;
+					testmat << node.matrix(0,0), node.matrix(0,1), node.matrix(1,0), node.matrix(1,1);
+					if (testmat.determinant() == 0) {
+						PRINT("Warning: Scaling a 2D object with 0 - removing object");
+						N.reset();
+					}
+					else {
+						CGAL_Aff_transformation2 t(
+							node.matrix(0,0), node.matrix(0,1), node.matrix(0,3),
+							node.matrix(1,0), node.matrix(1,1), node.matrix(1,3), node.matrix(3,3));
+						
+						DxfData *dd = N.convertToDxfData();
+						for (size_t i=0; i < dd->points.size(); i++) {
+							CGAL_Kernel2::Point_2 p = CGAL_Kernel2::Point_2(dd->points[i][0], dd->points[i][1]);
+							p = t.transform(p);
+							dd->points[i][0] = to_double(p.x());
+							dd->points[i][1] = to_double(p.y());
+						}
+						
+						PolySet ps;
+						ps.is2d = true;
+						dxf_tesselate(&ps, *dd, 0, true, false, 0);
+						
+						N = evaluateCGALMesh(ps);
+						delete dd;
+					}
+				}
+				else if (N.dim == 3) {
+					if (node.matrix.matrix().determinant() == 0) {
+						PRINT("Warning: Scaling a 3D object with 0 - removing object");
+						N.reset();
+					}
+					else {
+						CGAL_Aff_transformation t(
+							node.matrix(0,0), node.matrix(0,1), node.matrix(0,2), node.matrix(0,3),
+							node.matrix(1,0), node.matrix(1,1), node.matrix(1,2), node.matrix(1,3),
+							node.matrix(2,0), node.matrix(2,1), node.matrix(2,2), node.matrix(2,3), node.matrix(3,3));
+						N.p3->transform(t);
+					}
 				}
-				
-				PolySet ps;
-				ps.is2d = true;
-				dxf_tesselate(&ps, *dd, 0, true, false, 0);
-				
-				N = evaluateCGALMesh(ps);
-				delete dd;
-			}
-			else if (N.dim == 3) {
-				CGAL_Aff_transformation t(
-					node.matrix(0,0), node.matrix(0,1), node.matrix(0,2), node.matrix(0,3),
-					node.matrix(1,0), node.matrix(1,1), node.matrix(1,2), node.matrix(1,3),
-					node.matrix(2,0), node.matrix(2,1), node.matrix(2,2), node.matrix(2,3), node.matrix(3,3));
-				N.p3->transform(t);
 			}
 		}
 		else {
@@ -349,14 +382,17 @@ void CGALEvaluator::addToParent(const State &state, const AbstractNode &node, co
 	else {
 		// Root node, insert into cache
 		if (!isCached(node)) {
-			CGALCache::instance()->insert(this->tree.getIdString(node), N);
+			if (!CGALCache::instance()->insert(this->tree.getIdString(node), N)) {
+				PRINT("WARNING: CGAL Evaluator: Root node didn't fit into cache");
+			}
 		}
+		this->root = N;
 	}
 }
 
 CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const PolySet &ps)
 {
-	if (ps.empty()) return CGAL_Nef_polyhedron();
+	if (ps.empty()) return CGAL_Nef_polyhedron(ps.is2d ? 2 : 3);
 
 	if (ps.is2d)
 	{
@@ -424,15 +460,15 @@ CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const PolySet &ps)
 		struct PolyReducer
 		{
 			Grid2d<int> grid;
-			boost::unordered_map<std::pair<int,int>, std::pair<int,int> > edge_to_poly;
-			boost::unordered_map<int, CGAL_Nef_polyhedron2::Point> points;
-			typedef boost::unordered_map<int, std::vector<int> > PolygonMap;
+			std::map<std::pair<int,int>, std::pair<int,int> > edge_to_poly;
+			std::map<int, CGAL_Nef_polyhedron2::Point> points;
+			typedef std::map<int, std::vector<int> > PolygonMap;
 			PolygonMap polygons;
 			int poly_n;
 
 			void add_edges(int pn)
 			{
-				for (int j = 1; j <= this->polygons[pn].size(); j++) {
+				for (unsigned int j = 1; j <= this->polygons[pn].size(); j++) {
 					int a = this->polygons[pn][j-1];
 					int b = this->polygons[pn][j % this->polygons[pn].size()];
 					if (a > b) { a = a^b; b = a^b; a = a^b; }
@@ -447,7 +483,7 @@ CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const PolySet &ps)
 
 			void del_poly(int pn)
 			{
-				for (int j = 1; j <= this->polygons[pn].size(); j++) {
+				for (unsigned int j = 1; j <= this->polygons[pn].size(); j++) {
 					int a = this->polygons[pn][j-1];
 					int b = this->polygons[pn][j % this->polygons[pn].size()];
 					if (a > b) { a = a^b; b = a^b; a = a^b; }
@@ -494,11 +530,11 @@ CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const PolySet &ps)
 
 			int merge(int p1, int p1e, int p2, int p2e)
 			{
-				for (int i = 1; i < this->polygons[p1].size(); i++) {
+				for (unsigned int i = 1; i < this->polygons[p1].size(); i++) {
 					int j = (p1e + i) % this->polygons[p1].size();
 					this->polygons[this->poly_n].push_back(this->polygons[p1][j]);
 				}
-				for (int i = 1; i < this->polygons[p2].size(); i++) {
+				for (unsigned int i = 1; i < this->polygons[p2].size(); i++) {
 					int j = (p2e + i) % this->polygons[p2].size();
 					this->polygons[this->poly_n].push_back(this->polygons[p2][j]);
 				}
@@ -518,7 +554,7 @@ CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const PolySet &ps)
 					int poly1_n = work_queue.front();
 					work_queue.pop_front();
 					if (this->polygons.find(poly1_n) == this->polygons.end()) continue;
-					for (int j = 1; j <= this->polygons[poly1_n].size(); j++) {
+					for (unsigned int j = 1; j <= this->polygons[poly1_n].size(); j++) {
 						int a = this->polygons[poly1_n][j-1];
 						int b = this->polygons[poly1_n][j % this->polygons[poly1_n].size()];
 						if (a > b) { a = a^b; b = a^b; a = a^b; }
@@ -527,7 +563,7 @@ CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const PolySet &ps)
 							int poly2_n = this->edge_to_poly[std::pair<int,int>(a, b)].first +
 									this->edge_to_poly[std::pair<int,int>(a, b)].second - poly1_n;
 							int poly2_edge = -1;
-							for (int k = 1; k <= this->polygons[poly2_n].size(); k++) {
+							for (unsigned int k = 1; k <= this->polygons[poly2_n].size(); k++) {
 								int c = this->polygons[poly2_n][k-1];
 								int d = this->polygons[poly2_n][k % this->polygons[poly2_n].size()];
 								if (c > d) { c = c^d; d = c^d; c = c^d; }
@@ -557,7 +593,7 @@ CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const PolySet &ps)
 
 				BOOST_FOREACH(const PolygonMap::value_type &i, polygons) {
 					std::list<CGAL_Nef_polyhedron2::Point> plist;
-					for (int j = 0; j < i.second.size(); j++) {
+					for (unsigned int j = 0; j < i.second.size(); j++) {
 						int p = i.second[j];
 						plist.push_back(points[p]);
 					}
@@ -569,12 +605,7 @@ CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const PolySet &ps)
 		};
 
 		PolyReducer pr(ps);
-		int numpolygons_before = pr.polygons.size();
 		pr.reduce();
-		int numpolygons_after = pr.polygons.size();
-		if (numpolygons_after < numpolygons_before) {
-			PRINTF("reduce polygons: %d -> %d", numpolygons_before, numpolygons_after);
-		}
 		return CGAL_Nef_polyhedron(pr.toNef());
 #endif
 #if 0
@@ -621,8 +652,8 @@ CGAL_Nef_polyhedron CGALEvaluator::evaluateCGALMesh(const PolySet &ps)
 				N = new CGAL_Nef_polyhedron3(*P);
 			}
 		}
-		catch (CGAL::Assertion_exception e) {
-			PRINTF("CGAL error in CGAL_Nef_polyhedron3(): %s", e.what());
+		catch (const CGAL::Assertion_exception &e) {
+			PRINTB("CGAL error in CGAL_Nef_polyhedron3(): %s", e.what());
 		}
 		CGAL::set_error_behaviour(old_behaviour);
 		return CGAL_Nef_polyhedron(N);
