@@ -23,7 +23,7 @@
 #include <CGAL/Nef_S2/OGL_base_object.h>
 #include <CGAL/Simple_cartesian.h>
 #include <CGAL/Nef_3/SNC_decorator.h>
-#include <qgl.h>
+#include "system-gl.h"
 #include <cstdlib>
 
 #define CGAL_NEF3_MARKED_VERTEX_COLOR 183,232,92
