@@ -1,6 +1,8 @@
 #include "PolySetCGALEvaluator.h"
 #include "cgal.h"
 #include "cgalutils.h"
+#include <CGAL/convex_hull_3.h>
+
 #include "polyset.h"
 #include "CGALEvaluator.h"
 #include "projectionnode.h"
@@ -12,9 +14,109 @@
 #include "dxftess.h"
 #include "module.h"
 
+#include "svg.h"
 #include "printutils.h"
 #include "openscad.h" // get_fragments_from_r()
 #include <boost/foreach.hpp>
+#include <vector>
+
+/*
+
+ZRemover
+
+This class converts one or more already 'flat' Nef3 polyhedra into a Nef2
+polyhedron by stripping off the 'z' coordinates from the vertices. The
+resulting Nef2 poly is accumulated in the 'output_nefpoly2d' member variable.
+
+The 'z' coordinates will either be all 0s, for an xy-plane intersected Nef3,
+or, they will be a mixture of -eps and +eps, for a thin-box intersected Nef3.
+
+Notes on CGAL's Nef Polyhedron2:
+
+1. The 'mark' on a 2d Nef face is important when doing unions/intersections.
+ If the 'mark' of a face is wrong the resulting nef2 poly will be unexpected.
+2. The 'mark' can be dependent on the points fed to the Nef2 constructor.
+ This is why we iterate through the 3d faces using the halfedge cycle
+ source()->target() instead of the ordinary source()->source(). The
+ the latter can generate sequences of points that will fail the
+ the CGAL::is_simple_2() test, resulting in improperly marked nef2 polys.
+3. 3d facets have 'two sides'. we throw out the 'down' side to prevent dups.
+
+The class uses the 'visitor' pattern from the CGAL manual. See also
+http://www.cgal.org/Manual/latest/doc_html/cgal_manual/Nef_3/Chapter_main.html
+http://www.cgal.org/Manual/latest/doc_html/cgal_manual/Nef_3_ref/Class_Nef_polyhedron3.html
+OGL_helper.h
+*/
+
+class ZRemover {
+public:
+	logstream log;
+	CGAL_Nef_polyhedron2::Boundary boundary;
+	shared_ptr<CGAL_Nef_polyhedron2> tmpnef2d;
+	shared_ptr<CGAL_Nef_polyhedron2> output_nefpoly2d;
+	CGAL::Direction_3<CGAL_Kernel3> up;
+	ZRemover()
+	{
+		output_nefpoly2d.reset( new CGAL_Nef_polyhedron2() );
+		boundary = CGAL_Nef_polyhedron2::INCLUDED;
+		up = CGAL::Direction_3<CGAL_Kernel3>(0,0,1);
+		log = logstream(5);
+	}
+	void visit( CGAL_Nef_polyhedron3::Vertex_const_handle ) {}
+	void visit( CGAL_Nef_polyhedron3::Halfedge_const_handle ) {}
+	void visit( CGAL_Nef_polyhedron3::SHalfedge_const_handle ) {}
+	void visit( CGAL_Nef_polyhedron3::SHalfloop_const_handle ) {}
+	void visit( CGAL_Nef_polyhedron3::SFace_const_handle ) {}
+	void visit( CGAL_Nef_polyhedron3::Halffacet_const_handle hfacet ) {
+		log << " <!-- Halffacet visit. Mark: " << hfacet->mark() << " -->\n";
+		if ( hfacet->plane().orthogonal_direction() != this->up ) {
+			log << "  <!-- down-facing half-facet. skipping -->\n";
+			log << " <!-- Halffacet visit end-->\n";
+			return;
+		}
+
+		// possible optimization - throw out facets that are 'side facets' between
+		// the top & bottom of the big thin box. (i.e. mixture of z=-eps and z=eps)
+
+		CGAL_Nef_polyhedron3::Halffacet_cycle_const_iterator fci;
+		int contour_counter = 0;
+		CGAL_forall_facet_cycles_of( fci, hfacet ) {
+			if ( fci.is_shalfedge() ) {
+				CGAL_Nef_polyhedron3::SHalfedge_around_facet_const_circulator c1(fci), cend(c1);
+				std::vector<CGAL_Nef_polyhedron2::Explorer::Point> contour;
+				CGAL_For_all( c1, cend ) {
+					CGAL_Nef_polyhedron3::Point_3 point3d = c1->source()->target()->point();
+					CGAL_Nef_polyhedron2::Explorer::Point point2d( point3d.x(), point3d.y() );
+					contour.push_back( point2d );
+				}
+
+				if (contour.size()==0) continue;
+
+				log << " <!-- is_simple_2:" << CGAL::is_simple_2( contour.begin(), contour.end() ) << " --> \n";
+
+				tmpnef2d.reset( new CGAL_Nef_polyhedron2( contour.begin(), contour.end(), boundary ) );
+
+				if ( contour_counter == 0 ) {
+					log << " <!-- contour is a body. make union(). " << contour.size() << " points. -->\n" ;
+					*(output_nefpoly2d) += *(tmpnef2d);
+				} else {
+					log << " <!-- contour is a hole. make intersection(). " << contour.size() << " points. -->\n";
+					*(output_nefpoly2d) *= *(tmpnef2d);
+				}
+
+				log << "\n<!-- ======== output tmp nef: ==== -->\n"
+					<< OpenSCAD::dump_svg( *tmpnef2d ) << "\n"
+					<< "\n<!-- ======== output accumulator: ==== -->\n"
+					<< OpenSCAD::dump_svg( *output_nefpoly2d ) << "\n";
+
+				contour_counter++;
+			} else {
+				log << " <!-- trivial facet cycle skipped -->\n";
+			}
+		} // next facet cycle (i.e. next contour)
+		log << " <!-- Halffacet visit end -->\n";
+	} // visit()
+};
 
 PolySetCGALEvaluator::PolySetCGALEvaluator(CGALEvaluator &cgalevaluator)
 	: PolySetEvaluator(cgalevaluator.getTree()), cgalevaluator(cgalevaluator)
@@ -23,74 +125,99 @@ PolySetCGALEvaluator::PolySetCGALEvaluator(CGALEvaluator &cgalevaluator)
 
 PolySet *PolySetCGALEvaluator::evaluatePolySet(const ProjectionNode &node)
 {
+	//openscad_loglevel = 6;
+	logstream log(5);
+
 	// Before projecting, union all children
 	CGAL_Nef_polyhedron sum;
 	BOOST_FOREACH (AbstractNode * v, node.getChildren()) {
 		if (v->modinst->isBackground()) continue;
 		CGAL_Nef_polyhedron N = this->cgalevaluator.evaluateCGALMesh(*v);
 		if (N.dim == 3) {
-			if (sum.empty()) sum = N.copy();
+			if (sum.isNull()) sum = N.copy();
 			else sum += N;
 		}
 	}
-	if (sum.empty()) return NULL;
+	if (sum.isNull()) return NULL;
+	if (!sum.p3->is_simple()) {
+		if (!node.cut_mode) {
+			PRINT("WARNING: Body of projection(cut = false) isn't valid 2-manifold! Modify your design..");
+			return new PolySet();
+		}
+	}
 
-	PolySet *ps = new PolySet();
-	ps->convexity = node.convexity;
-	ps->is2d = true;
+	//std::cout << sum.dump();
+	//std::cout.flush();
 
-	if (node.cut_mode)
-	{
-		PolySet cube;
-		double infval = 1e8, eps = 0.1;
-		double x1 = -infval, x2 = +infval, y1 = -infval, y2 = +infval, z1 = 0, z2 = eps;
+	CGAL_Nef_polyhedron nef_poly(2);
 
-		cube.append_poly(); // top
-		cube.append_vertex(x1, y1, z2);
-		cube.append_vertex(x2, y1, z2);
-		cube.append_vertex(x2, y2, z2);
-		cube.append_vertex(x1, y2, z2);
-
-		cube.append_poly(); // bottom
-		cube.append_vertex(x1, y2, z1);
-		cube.append_vertex(x2, y2, z1);
-		cube.append_vertex(x2, y1, z1);
-		cube.append_vertex(x1, y1, z1);
-
-		cube.append_poly(); // side1
-		cube.append_vertex(x1, y1, z1);
-		cube.append_vertex(x2, y1, z1);
-		cube.append_vertex(x2, y1, z2);
-		cube.append_vertex(x1, y1, z2);
-
-		cube.append_poly(); // side2
-		cube.append_vertex(x2, y1, z1);
-		cube.append_vertex(x2, y2, z1);
-		cube.append_vertex(x2, y2, z2);
-		cube.append_vertex(x2, y1, z2);
-
-		cube.append_poly(); // side3
-		cube.append_vertex(x2, y2, z1);
-		cube.append_vertex(x1, y2, z1);
-		cube.append_vertex(x1, y2, z2);
-		cube.append_vertex(x2, y2, z2);
-
-		cube.append_poly(); // side4
-		cube.append_vertex(x1, y2, z1);
-		cube.append_vertex(x1, y1, z1);
-		cube.append_vertex(x1, y1, z2);
-		cube.append_vertex(x1, y2, z2);
-		CGAL_Nef_polyhedron Ncube = this->cgalevaluator.evaluateCGALMesh(cube);
-
-		// N.p3 *= CGAL_Nef_polyhedron3(CGAL_Plane(0, 0, 1, 0), CGAL_Nef_polyhedron3::INCLUDED);
-		sum *= Ncube;
-		if (!sum.p3->is_simple()) {
-			PRINTF("WARNING: Body of projection(cut = true) isn't valid 2-manifold! Modify your design..");
-			goto cant_project_non_simple_polyhedron;
+	if (node.cut_mode) {
+		CGAL::Failure_behaviour old_behaviour = CGAL::set_error_behaviour(CGAL::THROW_EXCEPTION);
+		try {
+			CGAL_Nef_polyhedron3::Plane_3 xy_plane = CGAL_Nef_polyhedron3::Plane_3( 0,0,1,0 );
+			*sum.p3 = sum.p3->intersection( xy_plane, CGAL_Nef_polyhedron3::PLANE_ONLY);
+		}
+		catch (const CGAL::Failure_exception &e) {
+			PRINTB("CGAL error in projection node during plane intersection: %s", e.what());
+			try {
+				PRINT("Trying alternative intersection using very large thin box: ");
+				std::vector<CGAL_Point_3> pts;
+				// dont use z of 0. there are bugs in CGAL.
+				double inf = 1e8;
+				double eps = 0.001;
+				CGAL_Point_3 minpt( -inf, -inf, -eps );
+				CGAL_Point_3 maxpt(  inf,  inf,  eps );
+				CGAL_Iso_cuboid_3 bigcuboid( minpt, maxpt );
+				for ( int i=0;i<8;i++ ) pts.push_back( bigcuboid.vertex(i) );
+				CGAL_Polyhedron bigbox;
+				CGAL::convex_hull_3( pts.begin(), pts.end(), bigbox );
+				CGAL_Nef_polyhedron3 nef_bigbox( bigbox );
+ 				*sum.p3 = nef_bigbox.intersection( *sum.p3 );
+			}
+			catch (const CGAL::Failure_exception &e) {
+				PRINTB("CGAL error in projection node during bigbox intersection: %s", e.what());
+				sum.p3->clear();
+			}
 		}
 
-		PolySet *ps3 = sum.convertToPolyset();
-		Grid2d<int> conversion_grid(GRID_COARSE);
+		if (sum.p3->is_empty()) {
+			CGAL::set_error_behaviour(old_behaviour);
+			PRINT("WARNING: projection() failed.");
+			return NULL;
+		}
+
+		// remove z coordinates to make CGAL_Nef_polyhedron2
+		log << OpenSCAD::svg_header( 480, 100000 ) << "\n";
+		try {
+			ZRemover zremover;
+			CGAL_Nef_polyhedron3::Volume_const_iterator i;
+			CGAL_Nef_polyhedron3::Shell_entry_const_iterator j;
+			CGAL_Nef_polyhedron3::SFace_const_handle sface_handle;
+			for ( i = sum.p3->volumes_begin(); i != sum.p3->volumes_end(); ++i ) {
+				log << "<!-- volume. mark: " << i->mark() << " -->\n";
+				for ( j = i->shells_begin(); j != i->shells_end(); ++j ) {
+					log << "<!-- shell. mark: " << i->mark() << " -->\n";
+					sface_handle = CGAL_Nef_polyhedron3::SFace_const_handle( j );
+					sum.p3->visit_shell_objects( sface_handle , zremover );
+					log << "<!-- shell. end. -->\n";
+				}
+				log << "<!-- volume end. -->\n";
+			}
+			nef_poly.p2 = zremover.output_nefpoly2d;
+		}	catch (const CGAL::Failure_exception &e) {
+			PRINTB("CGAL error in projection node while flattening: %s", e.what());
+		}
+		log << "</svg>\n";
+
+		CGAL::set_error_behaviour(old_behaviour);
+
+		// Extract polygons in the XY plane, ignoring all other polygons
+		// FIXME: If the polyhedron is really thin, there might be unwanted polygons
+		// in the XY plane, causing the resulting 2D polygon to be self-intersection
+		// and cause a crash in CGALEvaluator::PolyReducer. The right solution is to
+		// filter these polygons here. kintel 20120203.
+		/*
+		Grid2d<unsigned int> conversion_grid(GRID_COARSE);
 		for (size_t i = 0; i < ps3->polygons.size(); i++) {
 			for (size_t j = 0; j < ps3->polygons[i].size(); j++) {
 				double x = ps3->polygons[i][j][0];
@@ -111,17 +238,13 @@ PolySet *PolySetCGALEvaluator::evaluatePolySet(const ProjectionNode &node)
 			}
 		next_ps3_polygon_cut_mode:;
 		}
-		delete ps3;
+		*/
 	}
+	// In projection mode all the triangles are projected manually into the XY plane
 	else
 	{
-		if (!sum.p3->is_simple()) {
-			PRINTF("WARNING: Body of projection(cut = false) isn't valid 2-manifold! Modify your design..");
-			goto cant_project_non_simple_polyhedron;
-		}
-
 		PolySet *ps3 = sum.convertToPolyset();
-		CGAL_Nef_polyhedron np;
+		if (!ps3) return NULL;
 		for (size_t i = 0; i < ps3->polygons.size(); i++)
 		{
 			int min_x_p = -1;
@@ -160,22 +283,21 @@ PolySet *PolySetCGALEvaluator::evaluatePolySet(const ProjectionNode &node)
 					plist.push_back(p);
 			}
 			// FIXME: Should the CGAL_Nef_polyhedron2 be cached?
-			if (np.empty()) {
-				np.dim = 2;
-				np.p2.reset(new CGAL_Nef_polyhedron2(plist.begin(), plist.end(), CGAL_Nef_polyhedron2::INCLUDED));
+			if (nef_poly.isEmpty()) {
+				nef_poly.p2.reset(new CGAL_Nef_polyhedron2(plist.begin(), plist.end(), CGAL_Nef_polyhedron2::INCLUDED));
 			}
 			else {
-				(*np.p2) += CGAL_Nef_polyhedron2(plist.begin(), plist.end(), CGAL_Nef_polyhedron2::INCLUDED);
+				(*nef_poly.p2) += CGAL_Nef_polyhedron2(plist.begin(), plist.end(), CGAL_Nef_polyhedron2::INCLUDED);
 			}
 		}
 		delete ps3;
-		DxfData *dxf = np.convertToDxfData();
-		dxf_tesselate(ps, *dxf, 0, true, false, 0);
-		dxf_border_to_ps(ps, *dxf);
-		delete dxf;
 	}
 
-cant_project_non_simple_polyhedron:
+	PolySet *ps = nef_poly.convertToPolyset();
+	assert( ps != NULL );
+	ps->convexity = node.convexity;
+	logstream(9) << ps->dump() << "\n";
+
 	return ps;
 }
 
@@ -261,16 +383,18 @@ PolySet *PolySetCGALEvaluator::evaluatePolySet(const LinearExtrudeNode &node)
 		BOOST_FOREACH (AbstractNode * v, node.getChildren()) {
 			if (v->modinst->isBackground()) continue;
 			CGAL_Nef_polyhedron N = this->cgalevaluator.evaluateCGALMesh(*v);
-			if (N.dim != 2) {
-				PRINT("ERROR: linear_extrude() is not defined for 3D child objects!");
-			}
-			else {
-				if (sum.empty()) sum = N.copy();
-				else sum += N;
+			if (!N.isNull()) {
+				if (N.dim != 2) {
+					PRINT("ERROR: linear_extrude() is not defined for 3D child objects!");
+				}
+				else {
+					if (sum.isNull()) sum = N.copy();
+					else sum += N;
+				}
 			}
 		}
 
-		if (sum.empty()) return NULL;
+		if (sum.isNull()) return NULL;
 		dxf = sum.convertToDxfData();;
 	} else {
 		dxf = new DxfData(node.fn, node.fs, node.fa, node.filename, node.layername, node.origin_x, node.origin_y, node.scale);
@@ -302,15 +426,15 @@ PolySet *PolySetCGALEvaluator::extrudeDxfData(const LinearExtrudeNode &node, Dxf
 		if (dxf.paths[i].is_closed)
 			continue;
 		if (first_open_path) {
-			PRINTF("WARNING: Open paths in dxf_linear_extrude(file = \"%s\", layer = \"%s\"):",
-					node.filename.c_str(), node.layername.c_str());
+			PRINTB("WARNING: Open paths in dxf_linear_extrude(file = \"%s\", layer = \"%s\"):",
+					node.filename % node.layername);
 			first_open_path = false;
 		}
-		PRINTF("   %9.5f %10.5f ... %10.5f %10.5f",
-					 dxf.points[dxf.paths[i].indices.front()][0] / node.scale + node.origin_x,
-					 dxf.points[dxf.paths[i].indices.front()][1] / node.scale + node.origin_y, 
-					 dxf.points[dxf.paths[i].indices.back()][0] / node.scale + node.origin_x,
-					 dxf.points[dxf.paths[i].indices.back()][1] / node.scale + node.origin_y);
+		PRINTB("   %9.5f %10.5f ... %10.5f %10.5f",
+					 (dxf.points[dxf.paths[i].indices.front()][0] / node.scale + node.origin_x) %
+					 (dxf.points[dxf.paths[i].indices.front()][1] / node.scale + node.origin_y) %
+					 (dxf.points[dxf.paths[i].indices.back()][0] / node.scale + node.origin_x) %
+					 (dxf.points[dxf.paths[i].indices.back()][1] / node.scale + node.origin_y));
 	}
 
 
@@ -359,16 +483,18 @@ PolySet *PolySetCGALEvaluator::evaluatePolySet(const RotateExtrudeNode &node)
 		BOOST_FOREACH (AbstractNode * v, node.getChildren()) {
 			if (v->modinst->isBackground()) continue;
 			CGAL_Nef_polyhedron N = this->cgalevaluator.evaluateCGALMesh(*v);
-			if (N.dim != 2) {
-				PRINT("ERROR: rotate_extrude() is not defined for 3D child objects!");
-			}
-			else {
-				if (sum.empty()) sum = N.copy();
-				else sum += N;
+			if (!N.isNull()) {
+				if (N.dim != 2) {
+					PRINT("ERROR: rotate_extrude() is not defined for 3D child objects!");
+				}
+				else {
+					if (sum.isNull()) sum = N.copy();
+					else sum += N;
+				}
 			}
 		}
 
-		if (sum.empty()) return NULL;
+		if (sum.isNull()) return NULL;
 		dxf = sum.convertToDxfData();
 	} else {
 		dxf = new DxfData(node.fn, node.fs, node.fa, node.filename, node.layername, node.origin_x, node.origin_y, node.scale);
@@ -383,9 +509,9 @@ PolySet *PolySetCGALEvaluator::evaluatePolySet(const CgaladvNode &node)
 {
 	CGAL_Nef_polyhedron N = this->cgalevaluator.evaluateCGALMesh(node);
 	PolySet *ps = NULL;
-	if (!N.empty()) {
+	if (!N.isNull()) {
 		ps = N.convertToPolyset();
-		ps->convexity = node.convexity;
+		if (ps) ps->convexity = node.convexity;
 	}
 
 	return ps;
@@ -395,13 +521,13 @@ PolySet *PolySetCGALEvaluator::evaluatePolySet(const RenderNode &node)
 {
 	CGAL_Nef_polyhedron N = this->cgalevaluator.evaluateCGALMesh(node);
 	PolySet *ps = NULL;
-	if (!N.empty()) {
+	if (!N.isNull()) {
 		if (N.dim == 3 && !N.p3->is_simple()) {
-			PRINTF("WARNING: Body of render() isn't valid 2-manifold!");
+			PRINT("WARNING: Body of render() isn't valid 2-manifold!");
 		}
 		else {
 			ps = N.convertToPolyset();
-			ps->convexity = node.convexity;
+			if (ps) ps->convexity = node.convexity;
 		}
 	}
 	return ps;
@@ -430,15 +556,10 @@ PolySet *PolySetCGALEvaluator::rotateDxfData(const RotateExtrudeNode &node, DxfD
 		}
 
 		for (int j = 0; j < fragments; j++) {
-			double a = (j*2*M_PI) / fragments;
+			double a = (j*2*M_PI) / fragments - M_PI/2; // start on the X axis
 			for (size_t k = 0; k < dxf.paths[i].indices.size(); k++) {
-				if (dxf.points[dxf.paths[i].indices[k]][0] == 0) {
-					points[j][k][0] = 0;
-					points[j][k][1] = 0;
-				} else {
-					points[j][k][0] = dxf.points[dxf.paths[i].indices[k]][0] * sin(a);
-					points[j][k][1] = dxf.points[dxf.paths[i].indices[k]][0] * cos(a);
-				}
+				points[j][k][0] = dxf.points[dxf.paths[i].indices[k]][0] * sin(a);
+			 	points[j][k][1] = dxf.points[dxf.paths[i].indices[k]][0] * cos(a);
 				points[j][k][2] = dxf.points[dxf.paths[i].indices[k]][1];
 			}
 		}
