@@ -38,6 +38,8 @@
 #include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string.hpp>
 #include <algorithm>
+#include <sstream>
+#include <map>
 
 #include <QDir>
 #include "value.h"
@@ -77,7 +79,7 @@ DxfData::DxfData(double fn, double fs, double fa,
 
 	std::ifstream stream(filename.c_str());
 	if (!stream.good()) {
-		PRINTF("WARNING: Can't open DXF file `%s'.", filename.c_str());
+		PRINTB("WARNING: Can't open DXF file '%s'.", filename);
 		return;
 	}
 
@@ -139,9 +141,9 @@ DxfData::DxfData(double fn, double fs, double fa,
     try {
 		  id = boost::lexical_cast<int>(id_str);
     }
-    catch (boost::bad_lexical_cast &blc) {
+    catch (const boost::bad_lexical_cast &blc) {
 			if (!stream.eof()) {
-				PRINTF("WARNING: Illegal ID '%s' in `%s'", id_str.c_str(), filename.c_str());
+				PRINTB("WARNING: Illegal ID '%s' in `%s'", id_str % filename);
 			}
 			break;
   	}
@@ -237,7 +239,7 @@ DxfData::DxfData(double fn, double fs, double fa,
 				int n = get_fragments_from_r(r_major, fn, fs, fa);
 				n = (int)ceil(n * sweep_angle / (2 * M_PI));
 //				Vector2d p1;
-				Vector2d p1;
+				Vector2d p1; p1 << 0,0;
 				for (int i=0;i<=n;i++) {
 					double a = (ellipse_start_angle + sweep_angle*i/n);
 //					Vector2d p2(cos(a)*r_major, sin(a)*r_minor);
@@ -380,23 +382,23 @@ DxfData::DxfData(double fn, double fs, double fa,
 		}
     }
     catch (boost::bad_lexical_cast &blc) {
-	  	PRINTF("WARNING: Illegal value %s in `%s'", data.c_str(), filename.c_str());
+	  	PRINTB("WARNING: Illegal value %s in '%s'", data % filename);
   	}
 	}
 
 	BOOST_FOREACH(const EntityList::value_type &i, unsupported_entities_list) {
 		if (layername.empty()) {
-			PRINTF("WARNING: Unsupported DXF Entity `%s' (%x) in `%s'.",
-						 i.first.c_str(), i.second, QuotedString(QDir::current().relativeFilePath(QString::fromStdString(filename)).toStdString()).c_str());
+			PRINTB("WARNING: Unsupported DXF Entity '%s' (%x) in %s.",
+						 i.first % i.second % QuotedString(QDir::current().relativeFilePath(QString::fromStdString(filename)).toStdString()));
 		} else {
-			PRINTF("WARNING: Unsupported DXF Entity `%s' (%x) in layer `%s' of `%s'.",
-						 i.first.c_str(), i.second, layername.c_str(), QuotedString(QDir::current().relativeFilePath(QString::fromStdString(filename)).toStdString()).c_str());
+			PRINTB("WARNING: Unsupported DXF Entity '%s' (%x) in layer '%s' of %s.",
+						 i.first % i.second % layername % QuotedString(QDir::current().relativeFilePath(QString::fromStdString(filename)).toStdString()));
 		}
 	}
 
 	// Extract paths from parsed data
 
-	typedef boost::unordered_map<int, int> LineMap;
+	typedef std::map<int, int> LineMap;
 	LineMap enabled_lines;
 	for (size_t i = 0; i < lines.size(); i++) {
 		enabled_lines[i] = i;
@@ -518,7 +520,7 @@ void DxfData::fixup_path_direction()
 			break;
 		this->paths[i].is_inner = true;
 		double min_x = this->points[this->paths[i].indices[0]][0];
-		int min_x_point = 0;
+		size_t min_x_point = 0;
 		for (size_t j = 1; j < this->paths[i].indices.size(); j++) {
 			if (this->points[this->paths[i].indices[j]][0] < min_x) {
 				min_x = this->points[this->paths[i].indices[j]][0];
@@ -526,9 +528,9 @@ void DxfData::fixup_path_direction()
 			}
 		}
 		// rotate points if the path is in non-standard rotation
-		int b = min_x_point;
-		int a = b == 0 ? this->paths[i].indices.size() - 2 : b - 1;
-		int c = b == this->paths[i].indices.size() - 1 ? 1 : b + 1;
+		size_t b = min_x_point;
+		size_t a = b == 0 ? this->paths[i].indices.size() - 2 : b - 1;
+		size_t c = b == this->paths[i].indices.size() - 1 ? 1 : b + 1;
 		double ax = this->points[this->paths[i].indices[a]][0] - this->points[this->paths[i].indices[b]][0];
 		double ay = this->points[this->paths[i].indices[a]][1] - this->points[this->paths[i].indices[b]][1];
 		double cx = this->points[this->paths[i].indices[c]][0] - this->points[this->paths[i].indices[b]][0];
@@ -555,3 +557,27 @@ int DxfData::addPoint(double x, double y)
 	return this->points.size()-1;
 }
 
+std::string DxfData::dump() const
+{
+	std::stringstream out;
+	out << "DxfData"
+	  << "\n num points: " << points.size()
+	  << "\n num paths: " << paths.size()
+	  << "\n num dims: " << dims.size()
+	  << "\n points: ";
+	for ( size_t k = 0 ; k < points.size() ; k++ ) {
+		out << "\n  x y: " << points[k].transpose();
+	}
+	out << "\n paths: ";
+	for ( size_t i = 0; i < paths.size(); i++ ) {
+		out << "\n  path:" << i
+		  << "\n  is_closed: " << paths[i].is_closed
+		  << "\n  is_inner: " << paths[i].is_inner ;
+		DxfData::Path path = paths[i];
+		for ( size_t j = 0; j < path.indices.size(); j++ ) {
+			out << "\n  index[" << j << "]==" << path.indices[j];
+		}
+	}
+	out << "\nDxfData end";
+	return out.str();
+}
