@@ -69,6 +69,8 @@ GLView::GLView(const QGLFormat & format, QWidget *parent) : QGLWidget(format, pa
 	init();
 }
 
+static bool running_under_wine = false;
+
 void GLView::init()
 {
 	this->viewer_distance = 500;
@@ -100,6 +102,15 @@ void GLView::init()
 	static int sId = 0;
 	this->opencsg_id = sId++;
 #endif
+
+// see paintGL() + issue160 + wine FAQ
+#ifdef _WIN32
+#include <windows.h>
+	HMODULE hntdll = GetModuleHandle(L"ntdll.dll");
+	if (hntdll)
+		if ( (void *)GetProcAddress(hntdll, "wine_get_version") )
+			running_under_wine = true;
+#endif
 }
 
 void GLView::setRenderer(Renderer *r)
@@ -154,9 +165,9 @@ void GLView::initializeGL()
 														 "Extensions:\n"
 														 "%s\n",
 														 glewGetString(GLEW_VERSION),
+														 glGetString(GL_VERSION),
 														 glGetString(GL_RENDERER),
 														 glGetString(GL_VENDOR),
-														 glGetString(GL_VERSION),
 														 rbits, gbits, bbits, abits, dbits, sbits,
 														 glGetString(GL_EXTENSIONS));
 // FIXME: glGetString(GL_EXTENSIONS) is deprecated in OpenGL 3.0.
@@ -358,6 +369,7 @@ void GLView::setupPerspective()
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
 	glFrustum(-w_h_ratio, +w_h_ratio, -(1/w_h_ratio), +(1/w_h_ratio), +10.0, +FAR_FAR_AWAY);
+	gluLookAt(0.0, -viewer_distance, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0);
 }
 
 void GLView::setupOrtho(double distance, bool offset)
@@ -370,38 +382,35 @@ void GLView::setupOrtho(double distance, bool offset)
 	glOrtho(-w_h_ratio*l, +w_h_ratio*l,
 			-(1/w_h_ratio)*l, +(1/w_h_ratio)*l,
 			-FAR_FAR_AWAY, +FAR_FAR_AWAY);
+	gluLookAt(0.0, -viewer_distance, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0);
 }
 
 void GLView::paintGL()
 {
 	glEnable(GL_LIGHTING);
 
-	if (orthomode)
-		setupOrtho(viewer_distance);
+	if (orthomode) setupOrtho(viewer_distance);
+	else setupPerspective();
 
 	glMatrixMode(GL_MODELVIEW);
 	glLoadIdentity();
 
-	const QColor &bgcol = RenderSettings::inst()->color(RenderSettings::BACKGROUND_COLOR);
-	glClearColor(bgcol.redF(), bgcol.greenF(), bgcol.blueF(), 0.0);
+	Color4f bgcol = RenderSettings::inst()->color(RenderSettings::BACKGROUND_COLOR);
+	glClearColor(bgcol[0], bgcol[1], bgcol[2], 0.0);
 
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
 
-	gluLookAt(0.0, -viewer_distance, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0);
-
 	glRotated(object_rot_x, 1.0, 0.0, 0.0);
 	glRotated(object_rot_y, 0.0, 1.0, 0.0);
 	glRotated(object_rot_z, 0.0, 0.0, 1.0);
 
-	glTranslated(object_trans_x, object_trans_y, object_trans_z);
-
   // FIXME: Crosshairs and axes are lighted, this doesn't make sense and causes them
   // to change color based on view orientation.
 	if (showcrosshairs)
 	{
 		glLineWidth(3);
-		const QColor &col = RenderSettings::inst()->color(RenderSettings::CROSSHAIR_COLOR);
-		glColor3f(col.redF(), col.greenF(), col.blueF());
+		Color4f col = RenderSettings::inst()->color(RenderSettings::CROSSHAIR_COLOR);
+		glColor3f(col[0], col[1], col[2]);
 		glBegin(GL_LINES);
 		for (double xf = -1; xf <= +1; xf += 2)
 		for (double yf = -1; yf <= +1; yf += 2) {
@@ -412,6 +421,8 @@ void GLView::paintGL()
 		glEnd();
 	}
 
+	glTranslated(object_trans_x, object_trans_y, object_trans_z);
+
 	// Large gray axis cross inline with the model
   // FIXME: This is always gray - adjust color to keep contrast with background
 	if (showaxes)
@@ -451,8 +462,6 @@ void GLView::paintGL()
 
 		setupOrtho(1000,true);
 
-		gluLookAt(0.0, -1000, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0);
-
 		glMatrixMode(GL_MODELVIEW);
 		glLoadIdentity();
 		glRotated(object_rot_x, 1.0, 0.0, 0.0);
@@ -530,6 +539,8 @@ void GLView::paintGL()
 			fmodf(360 - object_rot_x + 90, 360), fmodf(360 - object_rot_y, 360), fmodf(360 - object_rot_z, 360), viewer_distance);
 		statusLabel->setText(msg);
 	}
+
+	if (running_under_wine) swapBuffers();
 }
 
 void GLView::keyPressEvent(QKeyEvent *event)
@@ -556,8 +567,6 @@ void GLView::mousePressEvent(QMouseEvent *event)
 {
 	mouse_drag_active = true;
 	last_mouse = event->globalPos();
-	grabMouse();
-	setFocus();
 }
 
 void GLView::normalizeAngle(GLdouble& angle)
@@ -591,14 +600,28 @@ void GLView::mouseMoveEvent(QMouseEvent *event)
 			normalizeAngle(object_rot_y);
 			normalizeAngle(object_rot_z);
 		} else {
-			// Right button pans
-			// Shift-right zooms
+			// Right button pans in the xz plane
+			// Middle button pans in the xy plane
+			// Shift-right and Shift-middle zooms
 			if ((QApplication::keyboardModifiers() & Qt::ShiftModifier) != 0) {
 				viewer_distance += (GLdouble)dy;
 			} else {
 
       double mx = +(dx) * viewer_distance/1000;
-      double my = -(dy) * viewer_distance/1000;
+      double mz = -(dy) * viewer_distance/1000;
+
+			double my = 0;
+#if (QT_VERSION < QT_VERSION_CHECK(4, 7, 0))
+			if (event->buttons() & Qt::MidButton) {
+#else
+			if (event->buttons() & Qt::MiddleButton) {
+#endif
+				my = mz;
+				mz = 0;
+				// actually lock the x-position
+				// (turns out to be easier to use than xy panning)
+				mx = 0;
+			}
 
 			Matrix3d aax, aay, aaz, tm3;
 			aax = Eigen::AngleAxisd(-(object_rot_x/180) * M_PI, Vector3d::UnitX());
@@ -614,15 +637,10 @@ void GLView::mouseMoveEvent(QMouseEvent *event)
 			Matrix4d vec;
 			vec <<
         0,  0,  0,  mx,
-        0,  0,  0,  0,
         0,  0,  0,  my,
+        0,  0,  0,  mz,
         0,  0,  0,  1
 			;
-			if ((QApplication::keyboardModifiers() & Qt::ShiftModifier) != 0) {
-        vec(0,3) = 0;
-        vec(1,3) = my;
-        vec(2,3) = 0;
-      }
       tm = tm * vec;
       object_trans_x += tm(0,3);
       object_trans_y += tm(1,3);
