@@ -5,6 +5,7 @@
 #include "grid.h"
 #include "linalg.h"
 #include <vector>
+#include <string>
 
 class PolySet
 {
@@ -24,21 +25,24 @@ public:
 	void append_poly();
 	void append_vertex(double x, double y, double z = 0.0);
 	void insert_vertex(double x, double y, double z = 0.0);
-
+	size_t memsize() const;
+	
 	BoundingBox getBoundingBox() const;
 
+#define CSGMODE_DIFFERENCE_FLAG 0x10
 	enum csgmode_e {
-		CSGMODE_NONE,
-		CSGMODE_NORMAL = 1,
-		CSGMODE_DIFFERENCE = 2,
-		CSGMODE_BACKGROUND = 11,
-		CSGMODE_BACKGROUND_DIFFERENCE = 12,
-		CSGMODE_HIGHLIGHT = 21,
-		CSGMODE_HIGHLIGHT_DIFFERENCE = 22
+		CSGMODE_NONE                  = 0x00,
+		CSGMODE_NORMAL                = 0x01,
+		CSGMODE_DIFFERENCE            = CSGMODE_NORMAL | CSGMODE_DIFFERENCE_FLAG,
+		CSGMODE_BACKGROUND            = 0x02,
+		CSGMODE_BACKGROUND_DIFFERENCE = CSGMODE_BACKGROUND | CSGMODE_DIFFERENCE_FLAG,
+		CSGMODE_HIGHLIGHT             = 0x03,
+		CSGMODE_HIGHLIGHT_DIFFERENCE  = CSGMODE_HIGHLIGHT | CSGMODE_DIFFERENCE_FLAG
 	};
 
 	void render_surface(csgmode_e csgmode, const Transform3d &m, GLint *shaderinfo = NULL) const;
 	void render_edges(csgmode_e csgmode) const;
+	std::string dump() const;
 };
 
 #endif
