@@ -33,14 +33,19 @@
 #include "handle_dep.h" // handle_dep()
 #include "visitor.h"
 
-#include <QFile>
-#include <QRegExp>
-#include <QStringList>
 #include <sstream>
+#include <fstream>
+#include <boost/foreach.hpp>
 #include <boost/unordered_map.hpp>
+#include <boost/tokenizer.hpp>
+#include <boost/lexical_cast.hpp>
+#include <boost/algorithm/string.hpp>
 #include <boost/assign/std/vector.hpp>
 using namespace boost::assign; // bring 'operator+=()' into scope
 
+#include <boost/filesystem.hpp>
+namespace fs = boost::filesystem;
+
 class SurfaceModule : public AbstractModule
 {
 public:
@@ -77,16 +82,17 @@ AbstractNode *SurfaceModule::evaluate(const Context *ctx, const ModuleInstantiat
 	Context c(ctx);
 	c.args(argnames, argexpr, inst->argnames, inst->argvalues);
 
-	node->filename = c.getAbsolutePath(c.lookup_variable("file").text);
+	Value fileval = c.lookup_variable("file");
+	node->filename = c.getAbsolutePath(fileval.isUndefined() ? "" : fileval.toString());
 
 	Value center = c.lookup_variable("center", true);
-	if (center.type == Value::BOOL) {
-		node->center = center.b;
+	if (center.type() == Value::BOOL) {
+		node->center = center.toBool();
 	}
 
 	Value convexity = c.lookup_variable("convexity", true);
-	if (convexity.type == Value::NUMBER) {
-		node->convexity = (int)convexity.num;
+	if (convexity.type() == Value::NUMBER) {
+		node->convexity = (int)convexity.toDouble();
 	}
 
 	return node;
@@ -95,10 +101,10 @@ AbstractNode *SurfaceModule::evaluate(const Context *ctx, const ModuleInstantiat
 PolySet *SurfaceNode::evaluate_polyset(class PolySetEvaluator *) const
 {
 	handle_dep(filename);
-	QFile f(QString::fromStdString(filename));
+	std::ifstream stream(filename.c_str());
 
-	if (!f.open(QIODevice::ReadOnly | QIODevice::Text)) {
-		PRINTF("WARNING: Can't open DAT file `%s'.", filename.c_str());
+	if (!stream.good()) {
+		PRINTB("WARNING: Can't open DAT file '%s'.", filename);
 		return NULL;
 	}
 
@@ -107,23 +113,34 @@ PolySet *SurfaceNode::evaluate_polyset(class PolySetEvaluator *) const
 	boost::unordered_map<std::pair<int,int>,double> data;
 	double min_val = 0;
 
-	while (!f.atEnd())
-	{
-		QString line = QString(f.readLine()).remove("\n").remove("\r");
-		line.replace(QRegExp("^[ \t]+"), "");
-		line.replace(QRegExp("[ \t]+$"), "");
+	typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
+	boost::char_separator<char> sep(" \t");
 
-		if (line.startsWith("#"))
-			continue;
-
-		QStringList fields = line.split(QRegExp("[ \t]+"));
-		for (int i = 0; i < fields.count(); i++) {
-			if (i >= columns)
-				columns = i + 1;
-			double v = fields[i].toDouble();
-			data[std::make_pair(lines, i)] = v;
-			min_val = fmin(v-1, min_val);
+	while (!stream.eof()) {
+		std::string line;
+		while (!stream.eof() && (line.size() == 0 || line[0] == '#')) {
+			std::getline(stream, line);
+			boost::trim(line);
 		}
+		if (stream.eof()) break;
+
+		int col = 0;
+		tokenizer tokens(line, sep);
+		try {
+			BOOST_FOREACH(const std::string &token, tokens) {
+				double v = boost::lexical_cast<double>(token);
+				data[std::make_pair(lines, col++)] = v;
+				if (col > columns) columns = col;
+				min_val = std::min(v-1, min_val);
+			}
+		}
+		catch (const boost::bad_lexical_cast &blc) {
+			if (!stream.eof()) {
+				PRINTB("WARNING: Illegal value in '%s': %s", filename % blc.what());
+			}
+			break;
+  	}
+
 		lines++;
 	}
 
@@ -208,9 +225,15 @@ PolySet *SurfaceNode::evaluate_polyset(class PolySetEvaluator *) const
 std::string SurfaceNode::toString() const
 {
 	std::stringstream stream;
+	fs::path path((std::string)this->filename);
 
 	stream << this->name() << "(file = " << this->filename << ", "
-		"center = " << (this->center ? "true" : "false") << ")";
+		"center = " << (this->center ? "true" : "false")
+#ifndef OPENSCAD_TESTING
+		// timestamp is needed for caching, but disturbs the test framework
+				 << ", " "timestamp = " << (fs::exists(path) ? fs::last_write_time(path) : 0)
+#endif
+				 << ")";
 
 	return stream.str();
 }
